/*
* file CcRolemap.java
*
* Licensed Materials - Property of IBM
* Restricted Materials of IBM
* 
* com.ibm.rational.wvcm.stp.cc.CcRolemap
*
* (C) Copyright IBM Corporation 2012.  All Rights Reserved. 
* Note to U.S. Government Users Restricted Rights:  Use, duplication or  
* disclosure restricted by GSA ADP  Schedule Contract with IBM Corp. 
*/
package com.ibm.rational.wvcm.stp.cc;

import static com.ibm.rational.wvcm.stpex.StpExBase.PROPERTY_NAMESPACE;

import java.util.List;
import java.util.Map;

import javax.wvcm.Feedback;
import javax.wvcm.Resource;
import javax.wvcm.PropertyNameList.PropertyName;
import javax.wvcm.WvcmException;

/**
 * <p>A proxy for a ClearCase rolemap.
 * 
 * <p>A rolemap specifies the principals (users, groups, etc) that take on roles listed in a 
 * policy.  In combination with a policy, it defines the access controls for one or more VOB objects. 
 * 
 * <p>The intention is that a small number of policies can be defined that determine how 
 * permissions are applied to objects. Defining a number of rolemaps for each policy 
 * describes who takes on the roles in the policy. By splitting the how and the who, 
 * policies can be reused multiple times to reduce the complexity of security administration.
 * 
 * Each VOB with access control lists (ACLs) enables has a default rolemap named <em>DefaultRolemap</em>.  
 * This default rolemap is empty. The default rolemap controls all objects in the VOB until new objects are 
 * created with a reference to some other rolemap, or until an existing object is reprotected 
 * to use a new rolemap.
 */

public interface CcRolemap extends CcTypeBase {
    
    /**
     * Map of names of VOB object proxy types to lists of access control entries which make up the 
     * effective ACLs that this rolemap would generate when applied to VOB objects.
     */
    PropertyName<Map<String, List<CcAccessControlEntry>>> ALL_EFFECTIVE_ACL =
        new PropertyName<Map<String, List<CcAccessControlEntry>>>(PROPERTY_NAMESPACE, "rolemap-all-eacl");

    /**
     * Get the value of this resource's {@link #ALL_EFFECTIVE_ACL} property.
     * 
     * @return Map of names of VOB object proxy types to lists of access control entries 
     * which make up the effective ACLs that this rolemap would generate when applied 
     * to VOB objects.
     * <code>null</code> if the resource does not support ACLs.
     * @throws WvcmException
     */
    Map<String, List<CcAccessControlEntry>> getAllEffectiveAcl() throws WvcmException;
  
    /**
     * Is this rolemap in use? <code>true</code> if the rolemap
     * is currently applied to one or more objects, <code>false</code> otherwise.
     */
    PropertyName<Boolean> IS_IN_USE = new PropertyName<Boolean>(PROPERTY_NAMESPACE, "rolemap-in-use");
    
    /**
     * Get the value of this resource's {@link #IS_IN_USE} property.
     * 
     * @return <code>true</code> if the rolemap is currently applied to one or 
     * more objects, <code>false</code> otherwise.
     * @throws WvcmException
     */
    boolean getIsInUse() throws WvcmException;
    
    /**
     * Policy used to define the ACLs for this rolemap.
     */
    PropertyName<CcPolicy> POLICY =
        new PropertyName<CcPolicy>(PROPERTY_NAMESPACE, "rolemap-policy");

    /**
     * Get the value of this resource's {@link #POLICY} property.
     * 
     * @return Policy used to define the ACLs for this rolemap.
     * @throws WvcmException
     */
    CcPolicy getPolicy() throws WvcmException;
    
    /**
     * Set the value of this proxy's {@link #POLICY} property.
     * This property may only be set for use with the 
     * {@link #doCreateCcRolemap(com.ibm.rational.wvcm.stp.cc.CcTypeBase.TypeCreateFlag[], Feedback)}
     * or {@link #doModifyCcRolemap(com.ibm.rational.wvcm.stp.cc.CcTypeBase.TypeCreateFlag[], Feedback)} 
     * calls.  It cannot be written using {@link Resource#doWriteProperties(Feedback)}.
     * @throws WvcmException
     *         if this proxy doesn't define a value for this property.
     */
    void setPolicy(CcPolicy policy) throws WvcmException;
  
    /**
     * List of role-to-principal bindings which define this rolemap
     */
    PropertyName<List<CcRolemapEntry>> ROLEMAP_ENTRIES =
        new PropertyName<List<CcRolemapEntry>>(PROPERTY_NAMESPACE, "rolemap-entries");

    /**
     * Get the value of this resource's {@link #ROLEMAP_ENTRIES} property.
     * 
     * @return List of role-to-principal bindings which define this rolemap
     * @throws WvcmException
     */
    List<CcRolemapEntry> getRolemapEntries() throws WvcmException;

    /**
     * Set the value of this proxy's {@link #ROLEMAP_ENTRIES} property.
     * This property may only be set for use with the 
     * {@link #doCreateCcRolemap(com.ibm.rational.wvcm.stp.cc.CcTypeBase.TypeCreateFlag[], Feedback)}
     * or {@link #doModifyCcRolemap(com.ibm.rational.wvcm.stp.cc.CcTypeBase.TypeCreateFlag[], Feedback)} calls.  
     * It cannot be written using {@link Resource#doWriteProperties(Feedback)}.
     * @throws WvcmException
     *         if this proxy doesn't define a value for this property.
     */
    void setRolemapEntries(List<CcRolemapEntry> rolemapEntries) throws WvcmException;
        
    /**
     * <p>
     * Create a new rolemap at the location specified by this proxy. The
     * location should be an object name selector specifying the rolemap's name
     * and the repository (VOB) in which to create it.
     * </p>
     * <p>
     * Set the required {@link #POLICY} property to specify the policy
     * with which the rolemap should be associated.
     * </p>
     * <p>
     * Set the optional {@link #ROLEMAP_ENTRIES} property to specify the role-to-principal bindings for
     * this rolemap.  If this property is not set, the rolemap will be created with an empty
     * binding list.
     * </p>
     * You may optionally set the following additional properties prior to
     * calling this method:
     * <bl>
     * <li> {@link #COMMENT} </li>
     * <li> {@link CcTypeBase#SCOPE} <em>(Not yet supported.)</em>
     * </bl>
     * @param flags Resource-specific creation flags. <em>(The ACQUIRE flag is not yet supported.)</em>
     */
    public CcRolemap doCreateCcRolemap(TypeCreateFlag[] flags, Feedback feedback) throws WvcmException;

    /**
     * <p>
     * Modify an existing rolemap at the location specified by this proxy. The
     * location should be an object name selector specifying the rolemap's name
     * and the repository (VOB) in which it is defined.
     * </p>
     * <p>
     * Set the optional {@link #POLICY} property to modify the policy
     * with which the rolemap should be associated.  If not specified, 
     * the existing policy will be left unchanged.
     * </p>
     * <p>
     * Set the optional {@link #ROLEMAP_ENTRIES} property to specify the role-to-principal bindings for
     * this rolemap.  If this property is not set, the existing rolemap entries
     * will be left unchanged.
     * </p>
     * You may optionally set the following additional properties prior to
     * calling this method:
     * <bl>
     * <li> {@link #COMMENT} Provides a comment for the rolemap modification, 
     * without overriding the creation comment.</li>
     * </bl>
     * @param flags Resource-specific modification flags. <em>(The ACQUIRE flag is not yet supported.)</em>
     */
    public CcRolemap doModifyCcRolemap(TypeCreateFlag[] flags, Feedback feedback) throws WvcmException;
}
